use std::collections::{VecDeque, HashSet};

use crate::challenge_args::ChallengeArgs;


type Position = Result<usize, ()>;


pub fn start_of_packet_pos(args: &ChallengeArgs) -> Position {
    return first_marker_pos(args, 4);
}

pub fn start_of_message_pos(args: &ChallengeArgs) -> Position {
    return first_marker_pos(args, 14);
}

fn first_marker_pos(args: &ChallengeArgs, window_size: usize) -> Position {
    let stream: &str = &args.input;
    let mut history: VecDeque<char> = Default::default();
    let mut pos: Result<usize, ()> = Err(());
    for (p, c) in stream.chars().enumerate() {

        history.push_back(c);

        for i in 0..(history.len().checked_sub(window_size).unwrap_or(0)) {
            history.pop_front();
            //history.drain(range);
        }
        assert!(history.len() <= window_size);

        // only try to match if the window is already filled
        if history.len() == window_size {
            // chars are unique to each other if a set filled with x chars has len==x
            let set: HashSet<&char> = HashSet::from_iter(history.iter());
            let is_unique: bool = set.len() == window_size;

            //dbg!("history: {}, set: {}, is_unique: {}", &history, set, is_unique);

            if is_unique {
                pos = Ok(p + 1); // requirements want it 1-based
                break;
            }
        }
    }
    pos
}