use std::{fs, io};
use clap::Parser;

use advent_of_code_2022_rust::{challenge_args::ChallengeArgs, day_01, day_02, day_06};


fn main() {
    let mut args: ChallengeArgs = ChallengeArgs::parse();
    read_file_if_needed(&mut args).expect("Could read not input from file.");
    solve_for_certain_day(&args);
}
fn read_file_if_needed(args: &mut ChallengeArgs) -> Result<(), io::Error> {
    if !args.input_as_path {
        return Ok(());
    }

    let file_content = fs::read_to_string(&args.input)?;
    args.input = file_content;

    Ok(())
}

fn solve_for_certain_day(args: &ChallengeArgs) {
    match args.day {
        1 => match args.task {
            1 => println!(
                "Most calories: {}",
                day_01::get_top_calorie(&args).expect("Error in parsing input")
            ),
            2 => println!(
                "Calories of top 3 elves: {}",
                day_01::get_total_cal_top(&args).expect("Error in parsing input")
            ),
            invalid_task => abort_for_invalid_task(invalid_task),
        },
        2 => {
            let total = day_02::calc_score_of_strat(args.task, &args.input);
            match args.task {
                1 => {
                    println!("The score following the strategy is: {}", total);
                }
                2 => {
                    println!("The score following the outcome strategy is: {}", total);
                }
                invalid_task => abort_for_invalid_task(invalid_task)
            }
        },
        // 3 => match args.task {
        //     1 => {
        //         let total = day_03::get_total_prio_of_dups(&args.input);
        //         println!("The total of priotities of the duplicates in the rucksacks: {total}");
        //     }
        //     2 => {
        //         let total = day_03::get_total_prio_of_group_badges(&args.input);
        //         println!("The total of prios of group badges: {total}");
        //     }
        //     invalid_task => abort_for_invalid_task(invalid_task),
        // },
        6 => {
            match args.task {
                1 => {
                    let pos = day_06::start_of_packet_pos(args);
                    match pos {
                        Ok(pos) => println!("Packet starts at: {}", pos),
                        Err(_) => println!("Packet: No marker found!")
                    }
                },
                2 => {
                    let pos = day_06::start_of_message_pos(args);
                    match pos {
                        Ok(pos) => println!("Message starts at: {}", pos),
                        Err(_) => println!("Message: No marker found!")
                    }
                }
                invalid_task => abort_for_invalid_task(invalid_task)
            }
        },
        unknown_day => abort_for_invalid_day(unknown_day),
    }
}


fn abort_for_invalid_day(unknown_day: u32) {
    eprint!("No solution for day with number: {unknown_day}");
    std::process::exit(1);
}

fn abort_for_invalid_task(unknown_task: u32) {
    eprint!("No solution for task with number: {unknown_task}");
    std::process::exit(2);
}